function TestModel(MODEL)

  global TestModel_global ;
  TestModel_global = MODEL ;

  close all ;

  x0   = feval(MODEL, 'init');
  epsi = 1E-10 ;

  %[x,g,radius] = TrustRegion( @fun, @Grad, @Hess, x0, epsi ) ;
  [x,g,radius] = DogLeg( @fun, @Grad, @Hess, x0, epsi ) ;
  plotIter( g, x, radius, sprintf('%s N.Iter=%d',feval(MODEL,'name'),length(g)) );

  x(:,end)
  g(end)
  disp('all done') ;
end

function res = fun( x )
  global TestModel_global ;
  res = feval(TestModel_global, 'f', x ) ;
end

function res = Grad( x )
  global TestModel_global ;
  res = feval(TestModel_global, 'Df', x ) ;
end

function res = Hess( x )
  global TestModel_global ;
  res = feval(TestModel_global, 'DDf', x ) ;
end

function plotIter( value, x, radius, tit )
  close all ;
  figure( 'Position', [10 10 400 400], 'color', 'w' ) ;
  semilogy(value, '--rs','LineWidth',2,...
                  'MarkerEdgeColor','k',...
                  'MarkerFaceColor','g',...
                  'MarkerSize',10) ;
  title(tit) ;
  hold off

  if length(x(:,1)) == 2
    global TestModel_global ;
    figure( 'Position', [500 10 400 400], 'color', 'w' ) ;
    hold on
    xmax = max(x(1,:)) ;
    xmin = min(x(1,:)) ;
    ymax = max(x(2,:)) ;
    ymin = min(x(2,:)) ;
    xc   = (xmax+xmin)/2 ;
    yc   = (ymax+ymin)/2 ;
    ray  = 0.1+0.7 * (max(xmax-xmin,ymax-ymin)+max(radius)) ;
    if ray ~= inf 
      PlotContour( @fun, xc, yc, ray ) ;
      plot(x(1,:),x(2,:), ...
           '-bo','LineWidth', 2,...
           'MarkerEdgeColor','k',...
           'MarkerFaceColor','r',...
           'MarkerSize',8 );
      for k=1:length(x(1,:))
        PlotDisc( x(1,k), x(2,k), radius(k));
      end
      title(tit) ;
    end
    hold off
  end

end


%
% Plot the MODEL (assumed by dimensional)
%
function PlotDisc( x, y, r )
  t = 0:0.1:2*pi ;
  xx = x + r * cos(t) ;
  yy = y + r * sin(t) ;
  plot(xx,yy,'-r') ;
end

%
% Plot the MODEL (assumed by dimensional)
%
function PlotContour( fun, x, y, ray )

  if ray > 0 & isfinite(ray) & isfinite(x) & isfinite(y)
    [X,Y] = meshgrid(x-ray:ray/100:x+ray, ...
                     y-ray:ray/100:y+ray) ;
    [nr,nc] = size(X) ;
    XY(1,:) = reshape(X,1,nr*nc) ;
    XY(2,:) = reshape(Y,1,nr*nc) ;
    ZZ      = feval( fun, XY ) ;
    Z       = reshape(ZZ,nr,nc) ;
    [C,h]   = contourf(X,Y,Z,40);
    axis equal tight ;
    %colormap cool ;
    colormap Jet ;
    %colormap Lines ;
  end
end
