function TestModel(MODEL)

  global TestModel_global ;
  TestModel_global = MODEL ;

  close all ;

  x0   = feval(MODEL, 'init');
  epsi = 1E-6 ;

  methods = {'SR1','DFP','BFGS', 'PSB'} ;
  for j=1:length(methods)
    ok = input( ['Start ' methods{j} ' [q=quit] '], 's' ) ;
    if ok == 'q' ; return ; end ;
    [f,x] = QuasiNewton( @fun, @Grad, x0, epsi, methods{j}, 'wolfe' ) ;
    plotIter( f, x, sprintf('%s N.Iter=%d Method=%s',feval(MODEL,'name'),length(f),methods{j}) );
  end

  ok = input( ['Start Newton [q=quit] '], 's' ) ;
  if ok == 'q'; return ; end ;
  [residual,x] = Newton( @F, @JF, x0, epsi ) ;
  plotIter( residual, x, sprintf('%s N.Iter=%d Method=Newton',feval(MODEL,'name'),length(residual))) ;

  ok = input( ['Start Newton [q=quit] '], 's' ) ;
  if ok == 'q'; return ; end ;
  [residual,x] = NewtonDamped( @F, @JF, x0, epsi, 'wolfe' ) ;
  plotIter( residual, x, sprintf('%s N.Iter=%d Method=NewtonDamped',feval(MODEL,'name'),length(residual))) ;
  
  ok = input( ['Start Broyden [q=quit] '], 's' ) ;
  if ok == 'q'; return ; end ;
  [residual,x] = Broyden( @F, x0, epsi ) ;
  plotIter( residual, x, sprintf('%s N.Iter=%d Method=Broyden',feval(MODEL,'name'),length(residual))) ;
  
  ok = input( ['Start Dumped Broyden [q=quit] '], 's' ) ;
  if ok == 'q'; return ; end ;
  [residual,x] = BroydenDamped( @F, x0, epsi, 'wolfe' ) ;
  plotIter( residual, x, sprintf('%s N.Iter=%d Method=BroydenDamped',feval(MODEL,'name'),length(residual))) ;

  methods = {'HS', 'PRP', 'LS', 'FR', 'CD', 'DY', 'HuS', 'TaS', 'LS-CD'};

  for j=1:length(methods)
    ok = input( ['Start ' methods{j} ' [q=quit] '], 's' ) ;
    if ok == 'q' ; return ; end ;
    [f,x] = NonlinearCG( @fun, @Grad, x0, epsi, methods{j}) ;
    plotIter( f, x, sprintf('%s N.Iter=%d Method=%s',feval(MODEL,'name'),length(f),methods{j}) );
  end

  disp('CG all done') ;
end

function res = fun( x )
  global TestModel_global ;
  res = feval(TestModel_global, 'f', x ) ;
end

function res = Grad( x )
  global TestModel_global ;
  res = feval(TestModel_global, 'Df', x ) ;
end

function res = F( x )
  global TestModel_global ;
  res = feval(TestModel_global, 'F', x ) ;
end

function res = JF( x )
  global TestModel_global ;
  res = feval(TestModel_global, 'JF', x ) ;
end

function plotIter( value, x, tit )
  close all ;
  figure( 'Position', [10 10 400 400], 'color', 'w' ) ;
  semilogy(value, '--rs','LineWidth',2,...
                  'MarkerEdgeColor','k',...
                  'MarkerFaceColor','g',...
                  'MarkerSize',10) ;
  title(tit) ;

  if length(x(:,1)) == 2
    global TestModel_global ;
    figure( 'Position', [500 10 400 400], 'color', 'w' ) ;
    hold on
    xmax = max(x(1,:)) ;
    xmin = min(x(1,:)) ;
    ymax = max(x(2,:)) ;
    ymin = min(x(2,:)) ;
    xc   = (xmax+xmin)/2 ;
    yc   = (ymax+ymin)/2 ;
    ray  = 0.1+0.7 * max(xmax-xmin,ymax-ymin) ;
    if ray ~= inf 
      PlotContour( @fun, xc, yc, ray ) ;
      plot(x(1,:),x(2,:), ...
           '-bo','LineWidth', 2,...
           'MarkerEdgeColor','k',...
           'MarkerFaceColor','r',...
           'MarkerSize',8 );
      title(tit) ;
    end
    hold off
  end

end

%
% Plot the MODEL (assumed by dimensional)
%
function PlotContour( fun, x, y, ray )

  if ray > 0 & isfinite(ray) & isfinite(x) & isfinite(y)
    [X,Y] = meshgrid(x-ray:ray/100:x+ray, ...
                     y-ray:ray/100:y+ray) ;
    [nr,nc] = size(X) ;
    XY(1,:) = reshape(X,1,nr*nc) ;
    XY(2,:) = reshape(Y,1,nr*nc) ;
    ZZ      = feval( fun, XY ) ;
    Z       = reshape(ZZ,nr,nc) ;
    [C,h]   = contourf(X,Y,sqrt(Z),40);
    axis equal tight ;
    %colormap cool ;
    colormap Jet ;
    %colormap Lines ;
  end
end
